//Создание задачи диспетчерского вызова прочих задач
function [EnabledTasks,..//Доступные задачи
          nowSolvingTasks,..//Решаемые в настоящий момент задачи
          err]=AskCommandPoint(AfterRegistrationTasksFunction,..//Функция, вызываемая после регистрации задач
                               ErrorFunction,..//Функция нештатных ситуаций
                               PointReadFunction,..//Функция получения сообщения
                               PointDisplayFunction,..//Функция отправки сообщения
                               IsRegisteredPointFunction,..//Функция проверки регистрации точки обмена сообщениями
                               SelectLaunchTaskForNode,..//Функция выбора задачи для запуска на текущем вычислительном узле
                               CommandPoint,..//Опрашиваемая точка обмена сообщениями
                               CommandPoints,..//Опрашиваемые точки обмена сообщениями
                               EnabledTasks,..//Доступные задачи
                               nowSolvingTasks,..//Решаемые в настоящий момент задачи
                               ProjectName,..//Имя проекта
                               tasksSplit,..//Разделитель доступных задач
                               pauseSeconds..//Время паузы (в секундах)
                               )
    //Проверяем аргументы
    nArg = argn(2);//Число аргументов
    if nArg < 11 then
        error('The function AskCommandPoint takes 11, 12 or 13 arguments');
    end
    select nArg//Число аргументов
    case 11 then
        tasksSplit = ',';//Разделитель доступных задач по умолчанию запятая
        pauseSeconds = 0;//Время паузы по умолчанию 1 секунда
    case 12 then
        if type(tasksSplit) == 1 then
            pauseSeconds = tasksSplit;
            tasksSplit = ',';//По умолчанию разделитель задач - запятые
        else
            pauseSeconds = 0;//По умолчанию берем одну секнду паузы
        end
    end
    
    //Проверяем корректность аргументов
    if (type(CommandPoint) ~= 10) ||..
       (size(CommandPoint,'*') ~= 1) then
        error('The argument CommandPoint of function RegisteringDispatchCommandPoint must be string scalar');
    end
    if type(CommandPoints) ~= 10 then
        error('The argument CommandPoints of function AskCommandPoints must be matrix of string');
    end
    if (~isempty(EnabledTasks)) && (type(EnabledTasks) ~= 10) then
        error('The argument EnabledTasks of function RegisteringDispatchCommandPoint must be matrix of string');
    end
    if (~isempty(nowSolvingTasks)) && (type(nowSolvingTasks) ~= 10) then
        error('The argument EnabledTasks of function RegisteringDispatchCommandPoint must be matrix of string');
    end
    if (type(ProjectName) ~= 10) ||..
       (size(ProjectName,'*') ~= 1) then
        error('The argument TaskInformationPoint of function SolveAllTasks must be string scalar');
    end
    if (type(tasksSplit) ~= 10) ||..
       (size(tasksSplit,'*') ~= 1) then
        error('The argument tasksSplit of function RegisteringDispatchCommandPoint must be string scalar');
    end
    if (type(pauseSeconds) ~= 1) ||..
       (size(pauseSeconds,'*') ~= 1) then
        error('The argument pauseSeconds of function RegisteringDispatchCommandPoint must be number scalar');
    end
    
    //Обрабатываем функцию считывания
    select type(PointReadFunction)
    case 10 then
        execstr('pointReadFunction = ' + PointReadFunction);
    case 15 then
        //Получаем имя функции отображения и ее параметры
        pRFunction = PointReadFunction(1);
        pRFunctionParametrs = PointReadFunction(2:$);
        
        //Приводим имя функции к типу функции
        execstr('pRFunction = ' + pRFunction);
        
        //Формируем функцию вызова
        function [msg,isQuension]=pointReadFunction(CommandPoint)
            [msg,isQuension] = pRFunction(CommandPoint,rFunctionParametrs(:));
        endfunction
    end
    
    //Обрабатываем функцию отображения
    select type(PointDisplayFunction)
    case 10 then
        execstr('pointDisplayFunction = ' + PointDisplayFunction);
    case 15 then
        //Получаем имя функции отображения и ее параметры
        pDFunction = PointDisplayFunction(1);
        pDFunctionParametrs = PointDisplayFunction(2:$);
        
        //Приводим имя функции к типу функции
        execstr('pDFunction = ' + pDFunction);
        
        //Формируем функцию вызова
        function [err]=pointDisplayFunction(msg,CommandPoint,pauseSeconds)
            err = pDFunction(msg,CommandPoint,pauseSeconds,..
                             pDFunctionParametrs(:));
        endfunction
    end
    
    //Обрабатываем функцию проверки зарегистрированности точки обмена сообщениями
    select type(IsRegisteredPointFunction)
    case 10 then
        execstr('isRegisteredPointFunction = ' + IsRegisteredPointFunction);
    case 15 then
        //Получаем имя функции отображения и ее параметры
        isRegPointFunction = IsRegisteredPointFunction(1);
        isRegPointFunctionParametrs = IsRegisteredPointFunction(2:$);
        
        //Приводим имя функции к типу функции
        execstr('isRegPointFunction = ' + isRegPointFunction);
        
        //Формируем функцию вызова
        function [regCode]=isRegisteredPointFunction(CommandPoint)
            regCode = isRegPointFunction(CommandPoint,..
                                         isRegPointFunctionParametrs(:));
        endfunction
    end
    
    //Обрабатываем функцию обработки ошибок
    select type(ErrorFunction)
    case 10 then
        execstr('errorFunction = ' + ErrorFunction);
    case 15 then
        //Получаем имя функции отображения и ее параметры
        errFunction = ErrorFunction(1);
        errFunctionParametrs = ErrorFunction(2:$);
        
        //Приводим имя функции к типу функции
        execstr('errFunction = ' + errFunction);
        
        //Формируем функцию вызова
        function errorFunction(regCode,errMsg,CommandPoint,ProjectName)
            errFunction(regCode,errMsg,CommandPoint,ProjectName,..
                        errFunctionParametrs(:));
        endfunction
    end
    
    //Обрабатываем функцию последующих действий после регистрации задач
    select type(AfterRegistrationTasksFunction)
    case 10 then
        execstr('afterRegistrationTasksFunction = ' + AfterRegistrationTasksFunction);
    case 15 then
        //Получаем имя функции отображения и ее параметры
        aftRegTasksFunction = AfterRegistrationTasksFunction(1);
        aftRegTasksFunctionParametrs = AfterRegistrationTasksFunction(2:$);
        
        //Приводим имя функции к типу функции
        execstr('aftRegTasksFunction = ' + aftRegTasksFunction);
        
        //Формируем функцию вызова
        function afterRegistrationTasksFunction(CommandPoint,Task,ProjectName)
            aftRegTasksFunction(CommandPoint,Task,ProjectName,..
                                aftRegTasksFunctionParametrs(:));
        endfunction
    end
    
    //Обрабатываем функцию выбора задачи для запуска на текущем вычислительном узле
    select type(SelectLaunchTaskForNode)
    case 10 then
        execstr('LaunchTask = ' + SelectLaunchTaskForNode);
    case 15 then
        //Получаем имя функции отображения и ее параметры
        lTask = SelectLaunchTaskForNode(1);
        lTaskParametrs = SelectLaunchTaskForNode(2:$);
        
        //Приводим имя функции к типу функции
        execstr('lTask = ' + lTask);
        
        //Формируем функцию вызова
        function [launchTask]=LaunchTask(enabledTasks,..
                                         commandPoints,..
                                         CommandPoint)
            launchTask = lTask(enabledTasks,commandPoints,..
                               CommandPoint,lTaskParametrs(:));
        endfunction
    end
    
    //Проверяем наличие точки обмена сообщениями
    regCode = isRegisteredPointFunction(CommandPoint);
    if regCode > 0 then
        //Обрабатываем ошибки
        nowSolvingTasks = ProcErrors(ProjectName,nowSolvingTasks,..
                                     CommandPoint,regCode,..
                                     errorFunction,..
                                     'This command point is not registered');
        
        //Выходим из функции
        err = -5;
        return;
    end
    
    //Считываем состояние точки обмена сообщениями
    [msg,..
     isQuension]=pointReadFunction(CommandPoint);
    if ~isQuension then
        //Обрабатываем ошибки
        RevNotRegisteredTask(ProjectName,nowSolvingTasks,..
                             CommandPoint,errorFunction,msg);
        
        //Выходим из функции
        err = -6;
        return;
    end
    if stripblanks(msg($)) ~= 'Current solving tasks:' then
        //Обрабатываем ошибки
        nowSolvingTasks = ProcErrors(ProjectName,nowSolvingTasks,..
                                     CommandPoint,4,errorFunction,msg);
        
        //Выходим из функции
        err = -7;
        return;
    end
    
    //Регистрируем уже решенную задачу и удаляем из текущих задач
    if size(msg,'*') > 1 then
        //Получаем уже решенную на текущем узле задачу
        [wasSolvedTask,..
         isEmptySolvedTask]=GetSolvingTaskOnNode(nowSolvingTasks,..
                                                 CommandPoint);
        
        //Регистрируем эту уже решенную задачу
        if ~isEmptySolvedTask then
            //Регистрируем задачу
            wasSolvedTaskMsg = msg($-1);//Сообщение уже решенной задачи
            err = RegisteredSolvedTaskFromRezult(wasSolvedTask,..
                                                 wasSolvedTaskMsg,..
                                                 ProjectName,%f);//Регистрируем задачу
            EnabledTasks = GetEnabledTasks(ProjectName,%f);//Обновляем доступные задачи
            if err == 0 then
                afterRegistrationTasksFunction(CommandPoint,..
                                               wasSolvedTask,..
                                               ProjectName);
            end
            
            //Удаляем решенную задачу из текущих задач
            nowSolvingTasks = DeleteNowSolvingTasks(ProjectName,..
                                                    nowSolvingTasks,..
                                                    CommandPoint);
            
            //Если задача не зарегистрирована, идем к следующему компу
            if err > 0 then
                errorFunction(-1,wasSolvedTask + ': ' + wasSolvedTaskMsg,..
                              CommandPoint,ProjectName);
                
                err = -27;
                return;
            end
        end
    end
    
    //Получаем доступные для запуска задачи
    [enabledTasks,..
     isEmptyNowEnabledTasks] = GetNowEnabledTasks(EnabledTasks,..
                                                  nowSolvingTasks);
    
    //Берем первую из доступных для запуска задач, ее и запустим
    if ~isEmptyNowEnabledTasks then
        //Получаем доступные (незанятые) вычислительные узлы
        commandPoints = GetNowEnabledCommandPoints(nowSolvingTasks,..
                                                   CommandPoints);
        
        //Задача для запуска
        launchTask = LaunchTask(enabledTasks,commandPoints,CommandPoint);
    
        //Запускаем задачу
        err = pointDisplayFunction(launchTask,CommandPoint,pauseSeconds);
        if err ~= 0 then
            //Обрабатываем ошибки
            nowSolvingTasks = ProcErrors(ProjectName,nowSolvingTasks,..
                                         CommandPoint,abs(err) + 4,..
                                         errorFunction,'Cannot send task to point');
            
            //Выходим из функции
            err = -8;
        else
            //Добавляем запущенную задачу в список уже запущенных задач
            nowSolvingTasks = AddingNowSolvingTasks(ProjectName,..
                                                    nowSolvingTasks,..
                                                    CommandPoint,..
                                                    launchTask);
        end
    end
    
    //Отсутствие ошибки
    err = 0;
endfunction

//Удаление решаемой сейчас задачи
function [nowSolvingTasks]=DeleteNowSolvingTasks(ProjectName,..
                                                 nowSolvingTasks,..
                                                 CommandPoint)
    //Получаем индексы задач
    [bDeletedTasksIndexes,..
     isEmptyTasks]=BDeletedTasksIndexes(nowSolvingTasks,..
                                        CommandPoint);
    if isEmptyTasks then
        return;
    end
    
    //Удаляем задачу из решаемых в настоящее время задач
    nowSolvingTasks(:,bDeletedTasksIndexes) = [];
            
    //Сохраняем решаемые в текущий момент времени задачи
    SavingNowSolvingTasks(ProjectName,nowSolvingTasks);
endfunction

//Добавление запущенной задачи в список задач
function [nowSolvingTasks]=AddingNowSolvingTasks(ProjectName,..
                                                 nowSolvingTasks,..
                                                 CommandPoint,..
                                                 AddingTask)
    //Получаем индексы задач
    [bDeletedTasksIndexes,..
     isEmptyTasks]=BDeletedTasksIndexes(nowSolvingTasks,..
                                        CommandPoint);
    
    //Добавляем запущенную задачу в список уже запущенных задач
    if isEmptyTasks then
        nowSolvingTasks = [nowSolvingTasks,[CommandPoint;AddingTask]];
    else
        nowSolvingTasks(2,bDeletedTasksIndexes) = AddingTask;
    end
    
    //Сохраняем решаемые в текущий момент времени задачи
    SavingNowSolvingTasks(ProjectName,nowSolvingTasks);
endfunction

//Индексы решаемых задач
function [bDeletedTasksIndexes,..
          isEmptyTasks]=BDeletedTasksIndexes(nowSolvingTasks,..
                                             CommandPoint)
    //Проверяем пустоту решаемых текущих задач
    if isempty(nowSolvingTasks) then
        bDeletedTasksIndexes = [];
        isEmptyTasks = %t;//Интересуемые задачи пустые
        return;
    end
    
    //Индексы удаляемых задач
    bDeletedTasksIndexes = (nowSolvingTasks(1,:) == CommandPoint);
    isEmptyTasks = (~or(bDeletedTasksIndexes));
endfunction

//Решаемая задача на текущем узле
function [solvingTask,..
          isEmptyTask]=GetSolvingTaskOnNode(nowSolvingTasks,..
                                            CommandPoint)//Получаем индексы задач
    //Проверяем, есть ли зарегистрированная задача на текущем узле
    [bDeletedTasksIndexes,..
     isEmptyTask]=BDeletedTasksIndexes(nowSolvingTasks,..
                                       CommandPoint);
    if isEmptyTask then
        solvingTask = [];
    else
        //Получаем задачу на текущем узле
        solvingTask = nowSolvingTasks(2,bDeletedTasksIndexes);
    end
endfunction

//Обработка незарегистрированности решаемой на текущем узле задачи
function RevNotRegisteredTask(ProjectName,..
                              nowSolvingTasks,..
                              CommandPoint,..
                              errFunction,msg)
    //Проверяем незаругистрированную задачу
    [bDeletedTasksIndexes,..
     isEmptyTasks]=BDeletedTasksIndexes(nowSolvingTasks,..
                                        CommandPoint);//Пустые или нет решаемые задачи
    if isEmptyTasks then
        ErrCode = 3;
    else
        ErrCode = 0;
    end
    errFunction(ErrCode,msg,CommandPoint,ProjectName);
endfunction

//Обработка ошибок
function [nowSolvingTasks]=ProcErrors(ProjectName,nowSolvingTasks,..
                                      CommandPoint,errCode,..
                                      errFunction,msg)
    //Удаляем комп из списка компов
    nowSolvingTasks = DeleteNowSolvingTasks(ProjectName,..
                                            nowSolvingTasks,..
                                            CommandPoint);

    //Обрабатываем ошибки
    errFunction(errCode,msg,CommandPoint,ProjectName);
endfunction

//Получение доступных задач
function [nowEnabledTasks,..
          isEmptyNowEnabledTasks]=GetNowEnabledTasks(nowEnabledTasks,..
                                                     nowSolvingTasks)
    //Отсеиваем уже решаемые задачи
    if (~isempty(nowSolvingTasks)) && (~isempty(nowEnabledTasks)) then
        nowNotEnabledTasksBInd = (members(nowEnabledTasks,nowSolvingTasks(2,:)) > 0);
        nowEnabledTasks(nowNotEnabledTasksBInd) = [];//Удаляем уже решаемые задачи
    end
    
    //Пустые или нет решаемые задачи
    isEmptyNowEnabledTasks = isempty(nowEnabledTasks);
endfunction

//Получение доступных точек обмена сообщениями
function [nowEnabledCommandPoints]=GetNowEnabledCommandPoints(nowSolvingTasks,..
                                                              nowEnabledCommandPoints)
    //Отсеиваем заданные командные точки
    if ~isempty(nowSolvingTasks) then
        bIndNowNotEnabledCommandPoints = (members(nowEnabledCommandPoints,..
                                                  nowSolvingTasks(1,:)) > 0);//Индексы уже занятых вычислительных узлов
        nowEnabledCommandPoints(bIndNowNotEnabledCommandPoints) = [];//Удаляем занятые точки
    end
endfunction
