//Создание задачи диспетчерского вызова прочих задач
function [EnabledTasks,..//Доступные задачи
          nowSolvingTasks..//Решаемые в настоящий момент задачи
          ]=AskCommandPoints(AfterRegistrationTasksFunction,..//Функция, вызываемая после регистрации задач
                             ErrorFunction,..//Функция нештатных ситуаций
                             DisplayFunction,..//Функция отправки сообщения
                             PointReadFunction,..//Функция получения сообщения
                             PointDisplayFunction,..//Функция отправки сообщения
                             IsRegisteredPointFunction,..//Функция проверки регистрации точки обмена сообщениями
                             SelectLaunchTaskForNode,..//Функция выбора задачи для запуска на текущем вычислительном узле
                             CommandPoints,..//Опрашиваемые точки обмена сообщениями
                             EnabledTasks,..//Доступные задачи
                             nowSolvingTasks,..//Решаемые в настоящий момент задачи
                             ProjectName,..//Имя проекта
                             tasksSplit,..//Разделитель доступных задач
                             pauseSeconds..//Время паузы (в секундах)
                             )
    //Проверяем аргументы
    nArg = argn(2);//Число аргументов
    if nArg < 11 then
        error('The function AskCommandPoints takes 11, 12 or 13 arguments');
    end
    select nArg//Число аргументов
    case 11 then
        tasksSplit = ',';//Разделитель доступных задач по умолчанию запятая
        pauseSeconds = 0;//Время паузы по умолчанию 1 секунда
    case 12 then
        if type(tasksSplit) == 1 then
            pauseSeconds = tasksSplit;
            tasksSplit = ',';//По умолчанию разделитель задач - запятые
        else
            pauseSeconds = 0;//По умолчанию берем одну секнду паузы
        end
    end
    
    //Проверяем корректность аргументов
    if type(CommandPoints) ~= 10 then
        error('The argument CommandPoints of function AskCommandPoints must be matrix of string');
    end
    if (~isempty(EnabledTasks)) && (type(EnabledTasks) ~= 10) then
        error('The argument EnabledTasks of function AskCommandPoints must be matrix of string');
    end
    if (~isempty(nowSolvingTasks)) && (type(nowSolvingTasks) ~= 10) then
        error('The argument EnabledTasks of function AskCommandPoints must be matrix of string');
    end
    if (type(ProjectName) ~= 10) ||..
       (size(ProjectName,'*') ~= 1) then
        error('The argument TaskInformationPoint of function AskCommandPoints must be string scalar');
    end
    if (type(tasksSplit) ~= 10) ||..
       (size(tasksSplit,'*') ~= 1) then
        error('The argument tasksSplit of function AskCommandPoints must be string scalar');
    end
    if (type(pauseSeconds) ~= 1) ||..
       (size(pauseSeconds,'*') ~= 1) then
        error('The argument pauseSeconds of function AskCommandPoints must be number scalar');
    end
    
    //Обрабатываем функцию отображения
    select type(DisplayFunction)
    case 10 then
        execstr('displayFunction = ' + DisplayFunction);
    case 15 then
        //Получаем имя функции отображения и ее параметры
        dFunction = DisplayFunction(1);
        dFunctionParametrs = DisplayFunction(2:$);
        
        //Приводим имя функции к типу функции
        execstr('dFunction = ' + dFunction);
        
        //Формируем функцию вызова
        function displayFunction(msg)
            dFunction(msg,dFunctionParametrs(:));
        endfunction
    end
    
    //Выводим на дисплей текущие доступные и решаемые задачи
    displayFunction('Asked command points:');
    
    //Опрашиваем в цикле точки обмена сообщениями
    for CommandPoint=matrix(CommandPoints,1,-1)
        //Приостанавливаем программу
        sleep(pauseSeconds, 's');
        
        //Выводим на дисплей опрашиваемую точку
        displayFunction('Asked command point ' + CommandPoint);
        
        //Опрашиваем точку
        [EnabledTasks,..//Доступные задачи
         nowSolvingTasks]=AskCommandPoint(AfterRegistrationTasksFunction,ErrorFunction,..
                                          PointReadFunction,PointDisplayFunction,..
                                          IsRegisteredPointFunction,..
                                          SelectLaunchTaskForNode,CommandPoint,..
                                          CommandPoints,EnabledTasks,nowSolvingTasks,..
                                          ProjectName,tasksSplit,pauseSeconds);
        
        //Выводим на дисплей
        DispEnabledTasks(EnabledTasks,tasksSplit,displayFunction);
        DispNowSolvingTasks(nowSolvingTasks,displayFunction);
    end
endfunction

//Вывод на экран доступных для решения задач
function DispEnabledTasks(EnabledTasks,tasksSplit,dispFunction)
    if isempty(EnabledTasks) then
        dispFunction('Enabled tasks: no');
    else
        dispFunction('Enabled tasks: ' + strcat(EnabledTasks,tasksSplit));
    end
endfunction

//Выводим на экран решаемые в текущий момент времени задачи
function DispNowSolvingTasks(nowSolvingTasks,dispFunction)
    if isempty(nowSolvingTasks) then
        dispFunction('Now solving tasks: no');
    else
        dispFunction('Now solving tasks: ');
        dispFunction(nowSolvingTasks');
    end
endfunction
