//Создание задачи диспетчерского вызова прочих задач
function [err]=EndCommandPoint(ErrorFunction,..//Функция нештатных ситуаций
                               PointReadFunction,..//Функция получения сообщения
                               PointDisplayFunction,..//Функция отправки сообщения
                               IsRegisteredPointFunction,..//Функция проверки регистрации точки обмена сообщениями
                               CommandPoint,..//Опрашиваемая точка обмена сообщениями
                               ProjectName,..//Имя проекта
                               pauseSeconds..//Время паузы (в секундах)
                               )
    //Проверяем аргументы
    nArg = argn(2);//Число аргументов
    if nArg < 6 then
        error('The function EndCommandPoint takes 6 or 7 arguments');
    elseif nArg == 6 then
        tasksSplit = ',';//Разделитель доступных задач по умолчанию запятая
    end
    
    //Проверяем корректность аргументов
    if (type(CommandPoint) ~= 10) ||..
       (size(CommandPoint,'*') ~= 1) then
        error('The argument CommandPoint of function RegisteringDispatchCommandPoint must be string scalar');
    end
    if (type(ProjectName) ~= 10) ||..
       (size(ProjectName,'*') ~= 1) then
        error('The argument TaskInformationPoint of function SolveAllTasks must be string scalar');
    end
    if (type(pauseSeconds) ~= 1) ||..
       (size(pauseSeconds,'*') ~= 1) then
        error('The argument pauseSeconds of function RegisteringDispatchCommandPoint must be number scalar');
    end
    
    //Обрабатываем функцию считывания
    select type(PointReadFunction)
    case 10 then
        execstr('pointReadFunction = ' + PointReadFunction);
    case 15 then
        //Получаем имя функции отображения и ее параметры
        pRFunction = PointReadFunction(1);
        pRFunctionParametrs = PointReadFunction(2:$);
        
        //Приводим имя функции к типу функции
        execstr('pRFunction = ' + pRFunction);
        
        //Формируем функцию вызова
        function [msg,isQuension]=pointReadFunction(CommandPoint)
            [msg,isQuension] = pRFunction(CommandPoint,rFunctionParametrs(:));
        endfunction
    end
    
    //Обрабатываем функцию отображения
    select type(PointDisplayFunction)
    case 10 then
        execstr('pointDisplayFunction = ' + PointDisplayFunction);
    case 15 then
        //Получаем имя функции отображения и ее параметры
        pDFunction = PointDisplayFunction(1);
        pDFunctionParametrs = PointDisplayFunction(2:$);
        
        //Приводим имя функции к типу функции
        execstr('pDFunction = ' + pDFunction);
        
        //Формируем функцию вызова
        function [err]=pointDisplayFunction(msg,CommandPoint,pauseSeconds)
            err = pDFunction(msg,CommandPoint,pauseSeconds,..
                             pDFunctionParametrs(:));
        endfunction
    end
    
    //Обрабатываем функцию проверки зарегистрированности точки обмена сообщениями
    select type(IsRegisteredPointFunction)
    case 10 then
        execstr('isRegisteredPointFunction = ' + IsRegisteredPointFunction);
    case 15 then
        //Получаем имя функции отображения и ее параметры
        isRegPointFunction = IsRegisteredPointFunction(1);
        isRegPointFunctionParametrs = IsRegisteredPointFunction(2:$);
        
        //Приводим имя функции к типу функции
        execstr('isRegPointFunction = ' + isRegPointFunction);
        
        //Формируем функцию вызова
        function [regCode]=isRegisteredPointFunction(CommandPoint)
            regCode = isRegPointFunction(CommandPoint,..
                                         isRegPointFunctionParametrs(:));
        endfunction
    end
    
    //Обрабатываем функцию обработки ошибок
    select type(ErrorFunction)
    case 10 then
        execstr('errorFunction = ' + ErrorFunction);
    case 15 then
        //Получаем имя функции отображения и ее параметры
        errFunction = ErrorFunction(1);
        errFunctionParametrs = ErrorFunction(2:$);
        
        //Приводим имя функции к типу функции
        execstr('errFunction = ' + errFunction);
        
        //Формируем функцию вызова
        function errorFunction(regCode,errMsg,CommandPoint,ProjectName)
            errFunction(regCode,errMsg,CommandPoint,ProjectName,..
                        errFunctionParametrs(:));
        endfunction
    end
    
    //Проверяем наличие точки обмена сообщениями
    regCode = isRegisteredPointFunction(CommandPoint);
    if regCode > 0 then
        //Обрабатываем ошибки
        errorFunction(12,'This command point ' + CommandPoint + ' is not registered',..
                      CommandPoint,ProjectName);
        
        //Выходим из функции
        err = -5;
        return;
    end
    
    //Считываем состояние точки обмена сообщениями
    [msg,..
     isQuension]=pointReadFunction(CommandPoint);
    if ~isQuension then
        //Обрабатываем ошибки
        errorFunction(12,msg,CommandPoint,ProjectName);
        
        //Выходим из функции
        err = -6;
        return;
    end
    if stripblanks(msg($)) ~= 'Current solving tasks:' then
        //Обрабатываем ошибки
        errorFunction(14,msg,CommandPoint,ProjectName);
        
        //Выходим из функции
        err = -7;
        return;
    end
    
    //Останавливаем обработку задач
    err = pointDisplayFunction('',CommandPoint,pauseSeconds);
    if err ~= 0 then
        //Обрабатываем ошибки
        errorFunction(12,'Cannot send stop message to point',CommandPoint,ProjectName);
        
        //Выходим из функции
        err = -8;
    end
endfunction
