//Создание задачи диспетчерского вызова прочих задач
function SolveAllTasks(AfterRegistrationTasksFunction,..//Функция, вызываемая после регистрации задач
                       ErrorFunction,..//Функция нештатных ситуаций
                       ReadFunction,..//Функция получения сообщения
                       DisplayFunction,..//Функция отправки сообщения
                       PointReadFunction,..//Функция получения сообщения
                       PointDisplayFunction,..//Функция отправки сообщения
                       IsRegisteredPointFunction,..//Функция проверки регистрации точки обмена сообщениями
                       SelectLaunchTaskForNode,..//Функция выбора задачи для запуска на текущем вычислительном узле
                       ProjectName,..//Имя проекта
                       tasksSplit,..//Разделитель доступных задач
                       pauseSeconds..//Время паузы (в секундах)
                       )
    //Проверяем аргументы
    nArg = argn(2);//Число аргументов
    if nArg < 9 then
        error('The function SolveAllTasks takes 9, 10 or 11 arguments');
    end
    select nArg//Число аргументов
    case 9 then
        tasksSplit = ',';//Разделитель доступных задач по умолчанию запятая
        pauseSeconds = 0;//Время паузы по умолчанию 1 секунда
    case 10 then
        if type(tasksSplit) == 1 then
            pauseSeconds = tasksSplit;
            tasksSplit = ',';//По умолчанию разделитель задач - запятые
        else
            pauseSeconds = 0;//По умолчанию берем одну секнду паузы
        end
    end
    
    //Проверяем корректность аргументов
    if (type(ProjectName) ~= 10) ||..
       (size(ProjectName,'*') ~= 1) then
        error('The argument TaskInformationPoint of function SolveAllTasks must be string scalar');
    end
    if (type(tasksSplit) ~= 10) ||..
       (size(tasksSplit,'*') ~= 1) then
        error('The argument tasksSplit of function SolveAllTasks must be string scalar');
    end
    if (type(pauseSeconds) ~= 1) ||..
       (size(pauseSeconds,'*') ~= 1) then
        error('The argument pauseSeconds of function SolveAllTasks must be number scalar');
    end
    
    //Обрабатываем функцию считывания
    select type(ReadFunction)
    case 10 then
        execstr('readFunction = ' + ReadFunction);
    case 15 then
        //Получаем имя функции отображения и ее параметры
        rFunction = ReadFunction(1);
        rFunctionParametrs = ReadFunction(2:$);
        
        //Приводим имя функции к типу функции
        execstr('rFunction = ' + rFunction);
        
        //Формируем функцию вызова
        function [msg]=readFunction(askStr)
            msg=rFunction(askStr,rFunctionParametrs(:));
        endfunction
    end
    
    //Создаем информационную точку
    CreateTaskInformationPoint(ProjectName);
    
    //Загружаем решаемые задачи
    nowSolvingTasks = LoadingNowSolvingTasks(ProjectName);
    
    //Получаем точки обмена сообщениями
    CommandPoints = GetCommandPoints();
    
    //Получаем доступные задачи
    EnabledTasks = GetEnabledTasks(ProjectName,%f);
    
    //Удаляем недоступные вычислительные узлы
    nowSolvingTasks = RemoveUnAvailableNodes(nowSolvingTasks,CommandPoints);
    
    //Перебираем в цикле точки обмена сообщениями
    msg = 'n';
    while ~or(msg == ['Y','y'])
        //Зацикливаем перебор точек обмена сообщениями
        while ~isempty(EnabledTasks)
            [EnabledTasks,..//Доступные задачи
             nowSolvingTasks..//Решаемые в настоящий момент задачи
             ]=AskCommandPoints(AfterRegistrationTasksFunction,ErrorFunction,..
                                DisplayFunction,PointReadFunction,..
                                PointDisplayFunction,IsRegisteredPointFunction,..
                                SelectLaunchTaskForNode,CommandPoints,..
                                EnabledTasks,nowSolvingTasks,ProjectName,..
                                tasksSplit,pauseSeconds);
        end
        
        //Спрашиваем, нужно ли выходить из цикла (может другие группы решают)
        msg = readFunction('Ia all tasks solved? (Y/n): ');
    end
    
    //Закрываем циклы обработки задач
    msg = readFunction('End tasks points? (Y/N): ');
    if ~or(msg == ['n','N']) then
        for CommandPoint=matrix(CommandPoints,1,-1)
            EndCommandPoint(ErrorFunction,PointReadFunction,..
                            PointDisplayFunction,IsRegisteredPointFunction,..
                            CommandPoint,ProjectName,pauseSeconds);
        end
    end
    
    //Удаляем точку обмена сообщениями
    DestroyTaskInformationPoint(ProjectName);
endfunction

//Удаление недоступных вычислительных узлов
function [nowSolvingTasks]=RemoveUnAvailableNodes(nowSolvingTasks,..
                                                  CommandPoints)
    if (~isempty(nowSolvingTasks)) && (~isempty(CommandPoints)) then
        bIndUnAvailableNodes = (members(nowSolvingTasks(1,:),CommandPoints) == 0);
        if or(bIndUnAvailableNodes) then
            nowSolvingTasks(:,bIndUnAvailableNodes) = [];
        end
    end
endfunction
